import path = require('path');
import {Binding, Component, CoreBindings, inject} from '@loopback/core';
import {RestApplication} from '@loopback/rest';
import {appinfo} from '@libit/pkginfo';
import {createTranslator} from '@libit/translate';
import * as swstats from 'swagger-stats';
import {FoundationConfig} from './types';
import {FoundationBindings} from './keys';
import {LoggerBindings, LoggerComponent} from './components/logger';
import {IncomingMessage} from 'http';

export class FoundationComponent implements Component {
  bindings: Binding[] = [];

  constructor(
    @inject(CoreBindings.APPLICATION_INSTANCE)
    private readonly app: RestApplication,
    @inject(FoundationBindings.CONFIG, {optional: true})
    private readonly options: FoundationConfig,
  ) {
    const [info, dir] = appinfo.sync(true);

    // Resolve options
    this.options = options ?? {};
    if (!this.options.projectRoot) {
      this.options.projectRoot = dir;
    }
    if (!this.options.projectName) {
      this.options.projectName = info.name;
    }
    if (!this.options.projectVersion) {
      this.options.projectVersion = info.version;
    }
    const {projectName} = this.options;

    // Mount logger component
    app.bind(LoggerBindings.CONFIG).to({
      name: projectName,
      ...options?.logger,
    });
    app.component(LoggerComponent);

    // Bind translator
    const resourceDir = this.options.translator?.resourceDir ?? path.join(__dirname, '..', 'intl');
    const translator = createTranslator('foundation', resourceDir);
    this.bindings.push(Binding.bind(FoundationBindings.TRANSLATOR).to(translator));

    // Enable OBF
    if (this.options?.obf && this.options?.obf.enabled !== false && this.options?.obf.openapiSpec) {
      const {obf} = this.options;
      const swStatsMiddleware = swstats.getMiddleware({
        uriPath: obf.uri ?? `/obf`,
        swaggerSpec: obf.openapiSpec,
        authentication: Boolean(obf.authentication),
        onAuthenticate: obf.authenticate
          ? obf.authenticate
          : (req: IncomingMessage, username: string, password: string) => {
              return username === obf.username && password === obf.password;
            },
      });
      app.expressMiddleware(FoundationBindings.SWSTATS_MIDDLEWARE, swStatsMiddleware);
    }
  }
}
